@@ -234,13 +234,14 @@ def main(args):
             L.append('\n\n')
 
             #write atom definition file
-            f= open(f"./runs/atom_{n_cyanos}_{n_ecw}_{SucPct}_{r}.in","w+")
+            f= open(f"runs/atom_{n_cyanos}_{n_ecw}_{SucPct}_{r}.in","w+")
             f.writelines(L)
-
-
-        #write initial conditions pickle file
-        dumpfile = open(f"./runs/run_{n_cyanos}_{n_ecw}_{SucPct}.pkl",'wb')
-        pickle.dump(InitialConditions,dumpfile)
+        RUN_DIR = Path(f'runs/Run_{n_cyanos}_{n_ecw}_{SucPct}_{args.reps}')
+        if not os.path.isdir(RUN_DIR):
+            os.mkdir(RUN_DIR)
+        #write initial conditions json file
+        dumpfile = open(RUN_DIR / 'metadata.json','w')
+        json.dump(InitialConditions, dumpfile, indent = 6)
         dumpfile.close()
         #write Inputscript
         #open the file
@@ -267,6 +268,7 @@ def main(args):
         z = int(InitialConditions['Dimensions'][2]*1e6)
         if args.datafed is True or args.datafed == 'True':
         #create DataFed collection to hold the results
+        # TODO actually make this work
             df_api = API()
             df_api.setContext('p/eng107')
             collectionName = f'NUFEB_{culture.n_cyanos}_{culture.n_ecw}_{culture.SucPct}_{x}_{y}_{z}'
