@@ -3,7 +3,7 @@ import h5py
 from pathlib import Path
 import pandas as pd
 import numpy as np
-from datafed.CommandLib import API
+import json
 import subprocess
 import sys
 import argparse
@@ -13,36 +13,57 @@ from urllib.parse import urlparse
 from urllib.request import urlretrieve
 import tarfile
 from nufeb_tools import __version__
-df_api = API()
-df_api.setContext('p/eng107')
+
 urls= ['https://github.com/Jsakkos/nufeb-tools/raw/main/data/runs.tar']
 
 class get_data:
-    """
+    """Collect results for analysis.
+
     NUFEB simulation data class to collect results for analysis
+
+    Attributes:
+        test (bool): Set `test = True` to get example data from the Github repository
+        directory (str): Path to the directory containing NUFEB simulation data. If `directory = None`, get_data will look for a DataFed collection
+        id (str):DataFed record ID, e.g., `"c/34558900"`
+        timestep (int): Length of simulation timestep in seconds
+        SucRatio (int): Relative cyanobacterial sucrose secretion level, 0-100
+        timepoints (List(str)): List of timepoints in the simulation
+        dims (List(str)): Size of the simulation boundaries in micrometers
+        numsteps (int): Number of timepoints
+        h5 (HDF5 File): HDF5 file containing NUFEB simulation data
+        biomass (pandas.DataFrame): Pandas Dataframe containing the biomass vs time data from biomass.csv
+        ntypes (pandas.DataFrame): Pandas Dataframe containing the cell number vs time data from ntypes.csv
+        avg_con (pandas.DataFrame): Pandas Dataframe containing the average nutrient concentrations vs time data from avg_concentration.csv
+        single_cell_biomass (pandas.DataFrame): Pandas Dataframe containing the single cell biomass over time of all cell ids present at the timepoint
+
     """
-    def __init__(self,directory,local=True,id=None,test=None,timestep=10):
+    def __init__(self,directory=None,id=None,test=None,timestep=10):
+        self.timestep=timestep
         if test:
             self.directory = str((Path.home()) / '.nufeb_tools' / 'data' / 'Run_26_90_83_1')
             if not os.path.isdir(self.directory):
-                download_test_data()
-            
-        else:
-            self.directory = directory
-        self.local = local
-        self.id = id
-        self.sucRatio = int(self.directory.split('_')[-2])
-        self.timestep=timestep
-        if self.local:
+                download_test_data()     
             self.get_local_data()
-        elif self.local is not True and self.id is not None:
-            self.datafed = self.get_datafed_data(self.id)
+        elif directory:
+            self.directory = directory
+            self.get_local_data()
+            self.sucRatio = int(self.directory.split('_')[-2])
+        elif id:
+            self.id = id
+            self.get_datafed_data()
+            self.sucRatio = int(self.directory.split('_')[3])
+        elif directory and id:
+            print('Define either a local directory or DataFed Collection ID, not both')
         else:
-            print('Something went wrong')
-        self.timepoints = [key for key in self.h5['concentration']['co2'].keys()]
-        self.timepoints.sort(key=int)
-        self.dims = self.h5['concentration']['co2']['0'].shape
-        self.numsteps = len(self.timepoints)
+            print('Missing local directory or DataFed Collection ID')
+        try:
+            self.timepoints = [key for key in self.h5['concentration']['co2'].keys()]
+            self.timepoints.sort(key=int)
+            self.dims = list(self.h5['concentration']['co2']['0'].shape)
+            self.dims += [self.dims.pop(0)] # move Z dimension to last: z,x,y to x,y,z
+            self.numsteps = len(self.timepoints)
+        except AttributeError:
+            print('Missing HDF5 file')
         
     def get_local_data(self):
         """
@@ -60,6 +81,9 @@ class get_data:
             delimiter='\t',
             names=['Time','O2','Sucrose','CO2'],
             skiprows=1)
+        f = open(os.path.join(self.directory,'metadata.json'),)
+        self.metadata  = json.load(f)
+        f.close()
         self.convert_units_avg_con()
         self.convert_units_biomass()
     def convert_units_avg_con(self):
@@ -76,21 +100,44 @@ class get_data:
         self.biomass.index = self.biomass.step/60/60*self.timestep
         self.biomass.index.name='Hours'
         self.biomass.iloc[:,1:]=self.biomass.iloc[:,1:]*1e18
-    def get_datafed_data(record_id):
+    def get_datafed_data(self,dir=None,orig_fname=True,wait=True):
         """
         Collect NUFEB simulation data from a DataFed collection
 
         Args:
-            record_id (str):
-                The DataFed record identifier of the data
+            dir (str):
+                Directory to download the DataFed collection to. Defaults to user/.nufeb/data/collection
+            orig_fname (bool):
+                Use original filenames
+            wait (bool):
+                Wait for the download to complete before moving on
         """
-        dv_resp = df_api.dataView(record_id)
-        get_resp = df_api.dataGet(record_id,
-                          '.', # directory where data should be downloaded
-                          orig_fname=False, # do not name file by its original name
-                          wait=True, # Wait until Globus transfer completes
-                         )
-        return get_resp
+        from datafed.CommandLib import API
+        df_api = API()
+        df_api.setContext('p/eng107')
+        if not dir:
+            dv_resp = df_api.collectionView(self.id)
+            self.directory = str((Path.home()) / '.nufeb_tools' / 'data' / dv_resp[0].coll[0].title)
+        get_resp = df_api.dataGet(self.id,
+                            path=self.directory,
+                            orig_fname=orig_fname,
+                            wait=wait,
+                            )
+        self.h5 = h5py.File(os.path.join(self.directory,'trajectory.h5'))
+        self.metadata = json.loads(dv_resp[0].data[0].metadata)
+        self.biomass = pd.read_csv(os.path.join(
+            self.directory,'biomass.csv'),
+                                   usecols=[0,1,2],delimiter='\t')
+        self.ntypes = pd.read_csv(os.path.join(
+            self.directory,'ntypes.csv'))
+        self.avg_con = pd.read_csv(os.path.join(
+            self.directory,'avg_concentration.csv'),
+            usecols=[0,2,3,4],
+            delimiter='\t',
+            names=['Time','O2','Sucrose','CO2'],
+            skiprows=1)
+        self.convert_units_avg_con()
+        self.convert_units_biomass()
     #print(dv_resp)
     def radius_key(self,timestep):
         """
@@ -112,7 +159,7 @@ class get_data:
 
         Args:
             timepoint (int):
-            The simulation timestep to calculate from. Default = 0.
+                The simulation timestep to calculate from. Default = 0.
         """
         self.hrs = [int(x)/360 for x in self.timepoints]
         df = pd.DataFrame(columns=['id','type','time','biomass'])
@@ -142,6 +189,97 @@ class get_data:
                 # append data to a dataframe
                 df = df.append(pd.DataFrame([[c,celltype,h,mass]],columns=['id','type','time','biomass']),ignore_index=True)
         self.single_cell_biomass = df
+    def get_positions(self,timepoint=0):
+        """
+        Extract the x, y, z position of each cell at a given timepoint
+
+        Args:
+            timepoint (int):
+                The simulation timestep to get the position data from.
+        
+        Returns:
+            df (pandas.DataFrame):
+                Dataframe containing ID, type,x, y, z columns
+        """
+        return pd.concat([pd.Series(self.h5['id'][str(timepoint)],name='ID'),
+        pd.Series(self.h5['type'][str(timepoint)],name='type'),
+        pd.Series(self.h5['x'][str(timepoint)],name='x'),
+        pd.Series(self.h5['y'][str(timepoint)],name='y'),
+        pd.Series(self.h5['z'][str(timepoint)],name='z')],axis=1)
+    def get_neighbor_distance(self,id,timepoint):
+        """
+        Get the nearest neighbor cell distances
+
+        Args:
+            id (int):
+                The ID of the reference cell
+            timepoint (int):
+                The timepoint to check the neighbor distances from
+        Returns:
+            df (pandas.DataFrame):
+                Dataframe containing ID, type, Distance
+        """
+        df = self.get_positions(timepoint)
+        temp = (df[df.ID ==id][['x','y','z']].squeeze() - df[df.ID !=id][['x','y','z']])**2
+        dist = pd.Series(np.sqrt(temp.x + temp.y + temp.z),name='Distance')
+        return pd.concat([df[df.ID !=id][['ID','type']],dist],axis=1).reset_index(drop=True)
+
+    def get_local_con(self,nutrient,timestep,cellID):
+        """
+        Get the local nutrient concentration of a cell
+
+        Args:
+            nutrient (str):
+                The nutrient to check the concentration of
+            timestep (int):
+                The timestep at which to check the concentration
+            cellID (int):
+                The cell identification number
+        
+        Returns:
+            Nutrient Concentration (float):
+                The concentration of the specified nutrient within the cell's grid
+        """
+        cell_locs = self.get_positions(timestep)
+        grid = [np.linspace(0,self.metadata['Dimensions'][x],self.dims[x]) for x in range(3)]
+        grid_loc = [get_grid_idx(grid[i],cell_locs[cell_locs.ID ==cellID][d].values[0]) for i,d in enumerate(['x','y','z'])]
+        if self.h5['concentration'].__contains__(nutrient):
+            return self.h5['concentration'][nutrient][str(timestep)][grid_loc[2],grid_loc[0],grid_loc[1]]
+        else:
+            nutes = list(self.h5['concentration'].__iter__())
+            print('Nutrient ' + nutrient + ' not found. Try:')
+            print(*nutes)
+
+def get_grid_idx(array,value):
+    """
+    Find the nutrient grid index value. Taken from https://stackoverflow.com/questions/2566412/find-nearest-value-in-numpy-array.
+
+    Args:
+        array (numpy.array):
+            1D Array containing the grid positions
+        value (float):
+            Cell location to map to the grid
+    Returns:
+        index (int):
+            Grid index
+    """
+    n = len(array)
+
+    jl = 0# Initialize lower
+    ju = n-1# and upper limits.
+    while (ju-jl > 1):# If we are not yet done,
+        jm=(ju+jl) >> 1# compute a midpoint with a bitshift
+        if (value >= array[jm]):
+            jl=jm# and replace either the lower limit
+        else:
+            ju=jm# or the upper limit, as appropriate.
+        # Repeat until the test condition is satisfied.
+    if (value == array[0]):# edge cases at bottom
+        return 0
+    elif (value == array[n-1]):# and top
+        return n-1
+    else:
+        return jl             
 def download_test_data(urls=urls):
     """
     Get an example dataset from the Github repo. Downloads to "home/.nufeb_tools/data"
@@ -166,12 +304,7 @@ def download_test_data(urls=urls):
             tar.close()
             Path(local_filename).unlink()
 
-def get_datafed(record_id,dir='.',orig_fname=False,wait=True):
-    get_resp = df_api.dataGet(record_id,
-                          path=dir,
-                          orig_fname=orig_fname,
-                          wait=wait,
-                         )
+
 
 def upload_datafed(file, title, collection_id,metadata_file):
     """
@@ -224,6 +357,7 @@ def create_datafed_collection(n_cyanos, n_ecw, SucPct,dims):
             x, y, z simulation boundaries
     """
     try:
+        from datafed.CommandLib import API
         df_api = API()
         df_api.setContext('p/eng107')
         collectionName = f'NUFEB_{n_cyanos}_{n_ecw}_{SucPct}_{dims[0]}_{dims[1]}_{dims[2]}'
@@ -243,7 +377,6 @@ def verify_datafed_connection():
     """
     
     try:
-        # This package is not part of anaconda and may need to be installed.
         from datafed.CommandLib import API
     except ImportError:
        # _logger.info('datafed not found. Installing from pip.')
